package vsp.uloha437;

import cz.zcu.fav.kiv.jsim.JSimInvalidParametersException;
import cz.zcu.fav.kiv.jsim.JSimProcess;
import cz.zcu.fav.kiv.jsim.JSimSecurityException;
import cz.zcu.fav.kiv.jsim.JSimSimulation;
import cz.zcu.fav.kiv.jsim.JSimSimulationAlreadyTerminatedException;
import cz.zcu.fav.kiv.jsim.JSimSystem;
import cz.zcu.fav.kiv.jsim.JSimTooManyHeadsException;
import cz.zcu.fav.kiv.jsim.JSimTooManyProcessesException;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Server pro zpracovani pozadavku.<br/>
 * <br/>
 * Doba obsluhy pozadavku ma exponencialni rozdeleni.
 *
 * @author Jiri Kucera (kalwi@students.zcu.cz)
 */
public class Server extends JSimProcess {

    private Queue queue;
    private Junction out;
    private double lambda;
    private double sum;

    /**
     * Konstruktor tridy.
     * 
     * @param name Nazev objektu.
     * @param simulation Reference na objekt simulacniho modelu.
     * @param lambda Parametr exponencialniho rozdeleni doby obsluhy pozadavku.
     * @param out Vystupni prvek.
     * @throws JSimSimulationAlreadyTerminatedException
     * @throws JSimInvalidParametersException
     * @throws JSimTooManyProcessesException
     * @throws JSimTooManyHeadsException
     * @throws Exception
     */
    public Server(String name, JSimSimulation simulation, double lambda, Junction out)
	    throws JSimSimulationAlreadyTerminatedException, JSimInvalidParametersException,
	    JSimTooManyProcessesException, JSimTooManyHeadsException, Exception {

	super(name, simulation);

	this.out = out;
	this.lambda = lambda;
	this.queue = new Queue(name + "-queue", simulation, this);
	this.sum = 0;
    }

    /**
     * Vrati frontu serveru, do ktere mohou byt vkladany pozadavky.
     * 
     * @return Fronta pridruzena k serveru.
     */
    public Queue getQueue() {
	return queue;
    }

    /**
     * Vrati prumerne zatizeni serveru.
     * @return Prumerne zatizeni serveru.
     */
    public double getRo() {
	return sum / myParent.getCurrentTime();
    }

    /**
     * Vrati prumerny pocet pozadavku v serveru.
     * @return Prumerny pocet pozadavku v serveru.
     */
    public double getLq() {
	return getRo() / (1 - getRo());
    }

    /**
     * V nekonecne smycce:
     * - ceka nahodnou dobu s exponencialnim rozdelenim
     * - vybere prvek z fronty
     * - vlozi prvek na vystup
     */
    @Override
    public void life() {
	try {
	    while (true) {
		if (queue.isEmpty()) {
		    passivate();
		} else {
		    double tmp = JSimSystem.negExp(lambda);
		    sum += tmp;
		    hold(tmp);

		    Transaction tr = queue.first();

		    out.put(tr);
		}
	    }
	}
	catch (JSimInvalidParametersException ex) {
	    Logger.getLogger(Server.class.getName()).log(Level.SEVERE, null, ex);
	}
	catch (JSimSecurityException ex) {
	    Logger.getLogger(Server.class.getName()).log(Level.SEVERE, null, ex);
	}
    }
}
